    function [xCG, plotresCG, ploterrCG] = ...
         CG(A, rhs, x0, n_max, tol, matvec, x_star)
%   function [xCG, plotresCG, ploterrCG] = ...
%       CG(A, rhs, x0, n_max, tol, matvec, x_star)
%   BF-10.09.04
%   computes the solution of Ax = rhs using the CG scheme.
% 
%   startvector:                      x0
%   maximal number of iterations:     n_max
%   desired accuracy of the residual: tol
%   solution:                         x_star
%
%   uses: matvec 
%

    %   initialize
    n               =   1;
    N               =   length(x0);
    xCG             =   x0;
    rCG             =   rhs - feval( matvec, A, xCG );
    rCGrCG          =   rCG' * rCG;
    rCGrCG_old      =   1;
    norm_rCG        =   sqrt(rCGrCG);
    norm_r0         =   norm_rCG;
    norm_e0         =   norm(x_star - x0);
    wCG             =   zeros(N, 1);

    plotresCG       =   zeros(n_max, 1);
    plotresCG(n)    =   log10(norm_rCG / norm_r0);
    ploterrCG       =   zeros(n_max, 1);
    ploterrCG(n)    =   log10( norm(  x_star  -  xCG  )  /  norm_e0 );


    %   iterate
    while (n < n_max + 1) &  (norm_rCG / norm_r0 > tol),

        n           =   n+1;
        nu          =   rCGrCG / rCGrCG_old;
        wCG         =   nu * wCG - rCG;
        AwCG        =   feval( matvec, A, wCG );
        eta         =   - rCGrCG / (wCG' * AwCG);

        %   update
        xCG         =   xCG + eta * wCG;
        rCG         =   rCG - eta * AwCG;

        rCGrCG_old  =   rCGrCG;
        rCGrCG      =   rCG' * rCG;
        %norm_rCG    =   sqrt(rCGrCG);
        norm_rCG    =   norm(rhs- feval( matvec, A, xCG ))


        plotresCG(n)=   log10( norm_rCG / norm_r0 );
        ploterrCG(n)=   log10( norm( x_star - xCG ) / norm_e0 );

        if rem(n,5) == 0,   disp( [n,  norm_rCG / norm_r0] );   end;

    end;    %   while

    plotresCG       =   plotresCG(1:n);
    ploterrCG       =   ploterrCG(1:n);

return; 
